@@ -1,5 +1,6 @@
 use crate::*;
 use std::collections::hash_map::Entry;
+
 /// Sorted Record storage.
 ///
 /// SortedFile is a tree of pages.
@@ -44,7 +45,7 @@ impl SortedFile {
             let p = &mut *pp.borrow_mut();
             if p.pnum != u64::MAX {
                 p.compress(db);
-                if false {
+                if true {
                     println!(
                         "Saving page {} root={} count={} node_size={} size={}",
                         p.pnum,
@@ -297,6 +298,19 @@ impl SortedFile {
                     pnum,
                 ));
                 e.insert(p.clone());
+
+                if false {
+                    let p = p.borrow();
+                    println!(
+                        "Loaded page {} root={} count={} node_size={} size={}",
+                        p.pnum,
+                        self.root_page,
+                        p.count,
+                        p.node_size,
+                        p.size()
+                    );
+                }
+
                 p
             }
         }
