@@ -289,20 +289,20 @@ impl Record for Zero {
 
 /// Helper class to read byte data using ColInfo.
 pub struct Access<'d, 'i> {
-    data: &'d [u8],
-    info: &'i ColInfo,
+    pub data: &'d [u8],
+    pub info: &'i ColInfo,
 }
 
 impl<'d, 'i> Access<'d, 'i> {
-    /// Extract int from byte data for column number colnum.
+    /// Extract int from byte data for specified column.
     pub fn int(&self, colnum: usize) -> i64 {
-        util::get(self.data, self.info.off[colnum], self.info.siz(colnum)) as i64
+        util::iget(self.data, self.info.off[colnum], self.info.siz(colnum)) as i64
     }
 
-    /// Extract string from byte data for column number colnum.
+    /// Extract string from byte data for specified column.
     pub fn str(&self, db: &DB, colnum: usize) -> String {
         let off = self.info.off[colnum];
-        let size = data_size(self.info.typ[colnum]);
+        let size = self.info.siz(colnum);
         let bytes = get_bytes(db, &self.data[off..], size).0;
         String::from_utf8(bytes).unwrap()
     }
@@ -315,8 +315,8 @@ impl<'d, 'i> Access<'d, 'i> {
 
 /// Helper class to write byte data using ColInfo.
 pub struct WriteAccess<'d, 'i> {
-    data: &'d mut [u8],
-    info: &'i ColInfo,
+    pub data: &'d mut [u8],
+    pub info: &'i ColInfo,
 }
 
 impl<'d, 'i> WriteAccess<'d, 'i> {
